// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IAnnotatableEntity;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.annotatedlineeditor.EntityListener;
import eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * <p>BulkEditorLineDisplay class.</p>
 *
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class BulkEditorLineDisplay implements ILineDisplayStrategy {
	
	private BulkEditor editor;

	/**
	 * <p>Constructor for BulkEditorLineDisplay.</p>
	 *
	 * @param editor a {@link org.eclipse.ui.IEditorPart} object.
	 */
	public BulkEditorLineDisplay(BulkEditor editor) {
		this.editor = editor;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getIcon(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public Image getIcon(Object entity) {
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getIndent(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public int getIndent(Object entity) {
		return 0;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getText(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public String getText(Object entity) {		
		return editor.getEditorInput().getText((ICdmBase) entity);
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#getSupplementalText(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public String getSupplementalText(Object entity) {
		
		String supplementalText = "";
		
		AbstractBulkEditorInput input = editor.getEditorInput();
		String typeText = input.getTypeText(entity);
		
		supplementalText += StringUtils.isBlank(typeText) ? "" : String.format(" [%s]", typeText);
		
		if (entity instanceof IAnnotatableEntity) {
			IAnnotatableEntity annotatableEntity = (IAnnotatableEntity) HibernateProxyHelper.deproxy(entity);

			Map<MarkerType, Boolean> editMarkerTypePreferences = PreferencesUtil.getEditMarkerTypePreferences();
			
			String markerText = "";
			List<String> markers = new ArrayList<String>();
			for (Marker marker : annotatableEntity.getMarkers()) {
				String markerLabel = marker.getMarkerType() == null? " unknown marker " : marker.getMarkerType().getLabel();
				markers.add(String.format("%1s = %2s", markerLabel, marker.getFlag() ? "yes" : "no"));
			}
			if (! markers.isEmpty()) {
				markerText = StringUtils.join(markers, ", ");
			}
			
			supplementalText += StringUtils.isBlank(markerText) ? "" : String.format(" [%s]", markerText);
		}
		
		return supplementalText;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#setText(java.lang.String, java.lang.Object)
	 */
	/** {@inheritDoc} */
	public void setText(String text, Object entity) {
		if (entity instanceof Reference) {
			((Reference) entity).setTitleCache(text);			
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#getHoverText(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public String getHoverText(Object entity) {
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ILineDisplayStrategy#setStatusMessage(java.lang.String, java.lang.Object)
	 */
	/** {@inheritDoc} */
	public void setStatusMessage(final String text, Object entity) {
		
		// Calling from the synchronized method ListEditorReconcilingStrategy#doReconcile
		// without its own thread causes an invalid thread access exception
		// see http://wiki.eclipse.org/FAQ_Why_do_I_get_an_invalid_thread_access_exception%3F
		new Thread(new Runnable() {
			public void run() {
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						IStatusLineManager statusLineManager = editor.getEditorSite().getActionBars().getStatusLineManager();
						statusLineManager.setMessage(text);
					}
				});
			}
		}).start();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#getEmptyText(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public String getEmptyCacheMessage(Object entity) {
		if (entity instanceof Reference) {
			return "No reference title cache";
		}
		if (entity instanceof TaxonNameBase) {
			return "No name title cache";
		}
		if (entity instanceof AgentBase) {
			return "No title cache";
		}
		return "";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#isEntityCacheEmpty(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public boolean isEntityCacheEmpty(Object entity) {
		if (entity instanceof Reference) {
			String text = ((Reference) entity).getTitleCache();
			return (text == null || text.equals(""));
		}
		if (entity instanceof TaxonNameBase) {
			String text = ((TaxonNameBase) entity).getTitleCache(); 
			return (text == null || text.equals(""));
		}
		if (entity instanceof AgentBase) {
			String text = ((AgentBase) entity).getTitleCache(); 
			return (text == null || text.equals(""));
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#addDisplayListener(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public void addDisplayListener(Object entity, EntityListener listener) {
		/* Note: all non-field specific listeners require that corresponding property sheets 
		 * contain the following:
		 * 
		 * 
		 * 		public void setPropertyValue(Object id, Object value) {
		 * 		...
		 * 			reference.firePropertyChange(new PropertyChangeEvent(reference, "", null, null));
		 * 		}
		 * 
		 */
		if (entity instanceof CdmBase) {
			((CdmBase) entity).addPropertyChangeListener(listener);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy#getFont(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public Font getFont(Object entity) {
		return null;
	}
}
